# -*- coding: utf-8 -*-

"""Test dexterity content type generation."""

from bobtemplates.plone import base
from bobtemplates.plone import content_type
from mrbob.bobexceptions import SkipQuestion
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator

import os
import pytest


def test_post_dexterity_type_name():
    """Test validation of entered dexterity type names."""

    def hookit(value):
        return content_type.check_dexterity_type_name(None, None, value)

    with pytest.raises(ValidationError):
        hookit("import")
    # Python 3.0 introduces additional characters from outside the ASCII range (see PEP 3131).
    # with pytest.raises(ValidationError):
    #     hookit("süpertype")
    #    with pytest.raises(ValidationError):
    #        hookit(u'Staff Member')
    with pytest.raises(ValidationError):
        hookit("2ndComing")
    #    with pytest.raises(ValidationError):
    #        hookit(u'Second Coming')
    with pytest.raises(ValidationError):
        hookit("*sterisk")
    with pytest.raises(ValidationError):
        hookit("da-sh")
    assert hookit("SuperType") == "SuperType"
    assert hookit("Super Type") == "Super Type"
    assert hookit("second_coming") == "second_coming"
    assert hookit("second coming") == "second coming"


def test_is_container_false():
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_base_class": "Item",
        },
    )
    with pytest.raises(SkipQuestion):
        content_type.is_container(configurator, None)


def test_is_container_true():
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_base_class": "Container",
        },
    )
    content_type.is_container(configurator, None)


def test_prepare_renderer(tmpdir):
    """Test prepare renderer."""
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "Special Task",
        },
    )
    content_type.prepare_renderer(configurator)
    assert configurator.variables["dexterity_type_name"] == "Special Task"
    assert (
        configurator.variables["dexterity_type_fti_file_name"] == "Special_Task"
    )  # NOQA: E501
    assert configurator.variables["dexterity_type_name_klass"] == "SpecialTask"
    assert (
        configurator.variables["dexterity_type_name_normalized"] == "special_task"
    )  # NOQA: E501
    assert configurator.target_directory.endswith(
        "/collective.todo/src/collective/todo"
    )  # NOQA: E501

    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "SpecialTask",
        },
    )
    content_type.prepare_renderer(configurator)
    assert configurator.variables["dexterity_type_name"] == "SpecialTask"
    assert (
        configurator.variables["dexterity_type_fti_file_name"] == "SpecialTask"
    )  # NOQA: E501
    assert configurator.variables["dexterity_type_name_klass"] == "SpecialTask"
    assert (
        configurator.variables["dexterity_type_name_normalized"] == "special_task"
    )  # NOQA: E501
    assert configurator.target_directory.endswith(
        "/collective.todo/src/collective/todo"
    )  # NOQA: E501

    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "special task",
        },
    )
    content_type.prepare_renderer(configurator)
    assert configurator.variables["dexterity_type_name"] == "special task"
    assert (
        configurator.variables["dexterity_type_fti_file_name"] == "special_task"
    )  # NOQA: E501
    assert configurator.variables["dexterity_type_name_klass"] == "SpecialTask"
    assert (
        configurator.variables["dexterity_type_name_normalized"] == "special_task"
    )  # NOQA: E501
    assert configurator.target_directory.endswith(
        "/collective.todo/src/collective/todo"
    )  # NOQA: E501

    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "Special_Task",
        },
    )
    content_type.prepare_renderer(configurator)
    assert configurator.variables["dexterity_type_name"] == "Special_Task"
    assert (
        configurator.variables["dexterity_type_fti_file_name"] == "Special_Task"
    )  # NOQA: E501
    assert configurator.variables["dexterity_type_name_klass"] == "SpecialTask"
    assert (
        configurator.variables["dexterity_type_name_normalized"] == "special_task"
    )  # NOQA: E501
    assert configurator.target_directory.endswith(
        "/collective.todo/src/collective/todo"
    )  # NOQA: E501


def test_check_global_allow_true():
    """Test global_allow set to True."""
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_global_allow": "y",
        },
    )
    with pytest.raises(SkipQuestion):
        content_type.check_global_allow(configurator, None)


def test_check_global_allow_false():
    """Test global_allow set to False."""
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_global_allow": "f",
            "dexterity_parent_container_type_name": "Folder",
        },
    )
    with pytest.raises(SkipQuestion):
        content_type.check_global_allow(configurator, None)


def test_update_parent_types_fti_xml(tmpdir):
    """Test xml changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    profiles_path = package_path + "/profiles/default/types"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path)
    template = """<?xml version="1.0"?>
<object xmlns:i18n="http://xml.zope.org/namespaces/i18n"
    name="My Parent"
    meta_type="Dexterity FTI"
    i18n:domain="collective.sample">
  <property name="global_allow">True</property>
  <property name="filter_content_types">True</property>
  <property name="allowed_content_types">
    <element value="child" />
  </property>
</object>
"""
    with open(os.path.join(profiles_path + "/My_Parent.xml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "child",
            "dexterity_type_global_allow": "f",
            "dexterity_parent_container_type_name": "My Parent",
        },
    )
    configurator.variables["package_folder"] = package_path
    content_type._update_parent_types_fti_xml(configurator)

    with open(os.path.join(profiles_path + "/My_Parent.xml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_update_rolemap_xml(tmpdir):
    """Test rolemap.xml changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    profiles_path = package_path + "/profiles/default"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path)
    template = """<?xml version="1.0"?>
<rolemap>
  <permissions>
  <!-- -*- extra stuff goes here -*- -->

    <permission name="collective.sample: Add Parent" acquire="True">
      <role name="Manager"/>
      <role name="Site Administrator"/>
      <role name="Owner"/>
      <role name="Contributor"/>
    </permission>
  </permissions>
</rolemap>
"""
    with open(os.path.join(profiles_path + "/rolemap.xml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "parent",
        },
    )
    configurator.variables["package_folder"] = package_path
    configurator.variables["package.dottedname"] = "bobtemplates.plone"
    configurator.variables["dexterity_type_name_klass"] = "Parent"
    content_type._update_rolemap_xml(configurator)

    with open(os.path.join(profiles_path + "/rolemap.xml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_update_permissions_zcml(tmpdir):
    """Test zcml changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """<configure
  xmlns="http://namespaces.zope.org/zope"
  xmlns:zcml="http://namespaces.zope.org/zcml"
  i18n_domain="plone">
  <configure zcml:condition="installed AccessControl.security">
  <!-- -*- extra stuff goes here -*- -->
    <permission
        id="collective.sample.AddParent"
        title="collective.sample: Add Parent"
    />
  </configure>
</configure>
"""
    with open(os.path.join(package_path + "/permissions.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:content_type",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "parent",
        },
    )
    configurator.variables["package_folder"] = package_path
    configurator.variables["package.dottedname"] = "bobtemplates.plone"
    configurator.variables["dexterity_type_name_klass"] = "Parent"
    content_type._update_permissions_zcml(configurator)

    with open(os.path.join(package_path + "/permissions.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_post_renderer(tmpdir):
    """Test post rendering."""
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    profiles_path = package_path + "/profiles/default"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path + "/types")

    template = """<?xml version="1.0" encoding="UTF-8"?>
<metadata>
  <version>1000</version>
  <dependencies>

  </dependencies>
</metadata>
"""
    with open(os.path.join(profiles_path + "/metadata.xml"), "w") as f:
        f.write(template)

    template = """<?xml version="1.0"?>
<rolemap>
  <permissions>
  <!-- -*- extra stuff goes here -*- -->

  </permissions>
</rolemap>
"""
    with open(os.path.join(profiles_path + "/rolemap.xml"), "w") as f:
        f.write(template)

    template = """<?xml version="1.0"?>
<object name="portal_types" meta_type="Plone Types Tool">
 <property name="title">Controls the available contenttypes in your portal
 </property>
 <!--<object name="example_ct" meta_type="Dexterity FTI"/>-->
</object>
"""
    with open(os.path.join(profiles_path + "/types.xml"), "w") as f:
        f.write(template)

    template = """<configure
  xmlns="http://namespaces.zope.org/zope"
  xmlns:zcml="http://namespaces.zope.org/zcml"
  i18n_domain="plone">

  <configure zcml:condition="installed AccessControl.security">
  <!-- -*- extra stuff goes here -*- -->


  </configure>

</configure>
"""
    with open(os.path.join(package_path + "/permissions.zcml"), "w") as f:
        f.write(template)

    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "dexterity_type_name": "Task",
            "plone.version": "5.1",
        },
    )

    os.chdir(package_path)
    base.set_global_vars(configurator)
    content_type.prepare_renderer(configurator)
    content_type.post_renderer(configurator)



# -*- coding: utf-8 -*-
"""Generate view."""

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import CONTENT_TYPE_INTERFACES
from bobtemplates.plone.base import echo
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import update_file
from bobtemplates.plone.base import ZCML_NAMESPACES
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort
from lxml import etree
from mrbob.bobexceptions import SkipQuestion
from mrbob.bobexceptions import ValidationError

import case_conversion as cc
import os
import six


def _view_name_from_python_class(configurator):
    view_generated_name = ""
    if configurator.variables["view_python_class"]:
        view_class_name = configurator.variables["view_python_class_name"]
        view_generated_name = cc.snakecase(view_class_name).replace("_", "-")
    return view_generated_name


def get_view_name_from_python_class(configurator, question):
    """Generate view default name from python class"""
    if configurator.variables["view_python_class"]:
        view_generated_name = _view_name_from_python_class(configurator)
        question.default = view_generated_name
    else:
        question.default = "my_view"


def get_view_template_name_from_python_class(configurator, question):
    if configurator.variables["view_python_class"]:
        view_class_name = configurator.variables["view_python_class_name"]
        view_generated_name = cc.snakecase(view_class_name)  # NOQA: E501
        question.default = view_generated_name
    else:
        question.default = "my_view"


def check_python_class_answer(configurator, question):
    if not configurator.variables["view_python_class"]:
        raise SkipQuestion(
            "No python class, so we skip python class name question."
        )  # NOQA: E501


def check_view_template_answer(configurator, question):
    if (
        not configurator.variables["view_template"]
        and not configurator.variables["view_python_class"]
    ):  # NOQA: E501
        raise ValidationError(
            "View must at least have a template or a python class"
        )  # NOQA: E501
    elif not configurator.variables["view_template"]:
        raise SkipQuestion(
            "No view template, so we skip view template name question."
        )  # NOQA: E501


def get_view_configuration(configurator):
    """return a dict with view configuration used for registration in zcml"""
    config = dict()
    config["name"] = configurator.variables["view_name"]
    # get Interface by content type or use the string it self as interface
    config["for"] = "{0}".format(
        CONTENT_TYPE_INTERFACES.get(
            configurator.variables["view_register_for"],
            configurator.variables["view_register_for"],
        )
    )
    if configurator.variables["view_template"]:
        config["template"] = "{0}.pt".format(
            configurator.variables["view_template_name"]
        )
    if configurator.variables["view_python_class"]:
        config["class"] = ".{0}.{1}".format(
            configurator.variables["view_python_file_name"],
            configurator.variables["view_python_class_name"],
        )
    # if configurator.variables["view_permission"]:
    config["permission"] = "zope2.View"
    return config


def _update_views_configure_zcml(configurator):
    file_name = "configure.zcml"
    directory_path = configurator.variables["package_folder"] + "/views/"
    file_path = directory_path + file_name
    configure_example_file_path = (
        configurator.variables["package_folder"] + "/views/configure.zcml.example"
    )
    file_list = os.listdir(os.path.dirname(directory_path))
    if file_name not in file_list:
        os.rename(configure_example_file_path, file_path)

    match_str = "-*- extra stuff goes here -*-"

    view_config = get_view_configuration(configurator)
    insert_str = """
  <browser:page
    name="{0}"
    for="{1}"
""".format(
        view_config["name"],
        view_config["for"],
    )
    if "class" in view_config:
        insert_str += '    class="{0}"\n'.format(view_config["class"])
    if "template" in view_config:
        insert_str += '    template="{0}"\n'.format(view_config["template"])
    if "permission" in view_config:
        insert_str += '    permission="{0}"\n'.format(view_config["permission"])
    insert_str += '    layer="{0}.interfaces.I{1}"\n'.format(
        configurator.variables["package.dottedname"],
        configurator.variables["package.browserlayer"],
    )
    insert_str += "    />\n"

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        view_name_xpath = "./browser:page[@name='{0}']".format(
            view_config["name"],
        )
        view_for_xpath = "./browser:page[@for='{0}']".format(
            view_config["for"],
        )
        if len(tree_root.xpath(view_name_xpath, namespaces=ZCML_NAMESPACES)) and len(
            tree_root.xpath(view_for_xpath, namespaces=ZCML_NAMESPACES)
        ):
            echo(
                "{0} already in configure.zcml, do you really want to add this config?"
                "\n\n{1}\n [y/N]: ".format(
                    configurator.variables["view_name"],
                    insert_str,
                ),
                "info",
            )
            if configurator.bobconfig.get("non_interactive"):
                return
            choice = six.moves.input().lower()
            if choice != "y":
                return

    update_file(configurator, file_path, match_str, insert_str)


def _update_configure_zcml(configurator):
    file_name = "configure.zcml"
    file_path = configurator.variables["package_folder"] + "/" + file_name
    namespaces = "{http://namespaces.zope.org/zope}"

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        view_xpath = "{0}include[@package='.views']".format(namespaces)
        if len(tree_root.findall(view_xpath)):
            print(
                ".views already in configure.zcml, skip adding!",
            )
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".views" />
"""
    update_file(configurator, file_path, match_str, insert_str)


def _delete_unwanted_files(configurator):
    directory_path = configurator.variables["package_folder"] + "/views/"
    if not configurator.variables["view_template"]:
        file_name = "{0}.pt".format(
            configurator.variables["view_template_name"],
        )
        file_path = directory_path + file_name
        os.remove(file_path)

    elif not configurator.variables["view_python_class"]:
        file_name = "{0}.py".format(
            configurator.variables["view_python_file_name"],
        )
        file_path = directory_path + file_name
        os.remove(file_path)

    file_name = "configure.zcml.example"
    file_list = os.listdir(os.path.dirname(directory_path))
    if file_name in file_list:
        file_path = directory_path + file_name
        os.remove(file_path)


def prepare_renderer(configurator):
    """Prepare rendering."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "view"
    view_name = configurator.variables["view_name"].strip("_")
    normalized_view_name = cc.snakecase(view_name)
    configurator.variables["view_name_normalized"] = normalized_view_name
    if configurator.variables["view_python_class"]:
        python_class_name = configurator.variables["view_python_class_name"].strip(
            "_"
        )  # NOQA: E501
        configurator.variables["view_python_class_name"] = cc.pascalcase(  # NOQA: E501
            python_class_name,
        )
        view_python_file_name = cc.snakecase(python_class_name)
        configurator.variables["view_python_file_name"] = view_python_file_name
        view_name_from_input = normalized_view_name.replace("_", "-")
        view_name_from_python_class = view_python_file_name.replace("_", "-")
        if view_name_from_input != view_name_from_python_class:
            configurator.variables["view_name"] = view_name_from_input
    else:
        configurator.variables["view_python_file_name"] = normalized_view_name

    if not configurator.variables["view_template"]:
        configurator.variables["view_template_name"] = normalized_view_name

    if not configurator.variables.get("view_base_class"):
        configurator.variables["view_base_class"] = "BrowserView"

    configurator.target_directory = configurator.variables["package_folder"]


def post_renderer(configurator):
    """Post rendering."""
    _update_configure_zcml(configurator)
    _update_views_configure_zcml(configurator)
    _delete_unwanted_files(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_commit(
        configurator,
        "Add view: {0}".format(
            configurator.variables["view_name"],
        ),
    )



# -*- coding: utf-8 -*-

from .base import init_package_base_structure
from bobtemplates.plone import base
from bobtemplates.plone import upgrade_step
from mrbob.configurator import Configurator

import os


def test_pre_renderer(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)
    target_path = os.path.join(package_path, "profiles/default")

    template = """<?xml version='1.0' encoding='UTF-8'?>
<metadata>
  <version>1004</version>
  <dependencies>
    <dependency>profile-plone.app.dexterity:default</dependency>
  </dependencies>
</metadata>
"""
    with open(os.path.join(target_path, "metadata.xml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:upgrade_step",
        target_directory=package_path,
        variables={
            "package_folder": package_path,
            "upgrade_step_title": "Add cool index and reindex it",
            "upgrade_step_description": "We add an index and reindex it with existing content.",
        },
    )
    assert configurator
    upgrade_step.pre_renderer(configurator)
    assert configurator.variables["upgrade_step_source_version"] == 1004
    assert configurator.variables["upgrade_step_dest_version"] == 1005
    assert configurator.variables["upgrade_step_id"] == "1005"


def test_post_renderer(tmpdir):
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    profiles_path = package_path + "/profiles/default"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path)

    template = """<?xml version="1.0" encoding="UTF-8"?>
<metadata>
  <version>1000</version>
  <dependencies>

  </dependencies>
</metadata>
"""
    with open(os.path.join(profiles_path + "/metadata.xml"), "w") as f:
        f.write(template)

    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">

    <!-- -*- extra stuff goes here -*- -->

    </configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:upgrade_step",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "plone.version": "5.1",
            "upgrade_step_title": "Add cool index and reindex it",
            "upgrade_step_description": "We add an index and reindex it with existing content.",
        },
    )

    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()


def test_read_source_version(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)
    target_path = os.path.join(package_path, "profiles/default")

    template = """<?xml version='1.0' encoding='UTF-8'?>
<metadata>
  <version>1004</version>
  <dependencies>
    <dependency>profile-plone.app.dexterity:default</dependency>
  </dependencies>
</metadata>
"""
    with open(os.path.join(target_path, "metadata.xml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:upgrade_step",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "plone.version": "5.1",
            "package_folder": package_path,
            "upgrade_step_title": "Add cool index and reindex it",
            "upgrade_step_description": "We add an index and reindex it with existing content.",
        },
    )
    assert configurator
    assert upgrade_step._read_source_version(configurator) == 1004


def test_write_dest_version(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)
    target_path = os.path.join(package_path, "profiles/default")

    template = """<?xml version='1.0' encoding='UTF-8'?>
<metadata>
  <version>1004</version>
  <dependencies>
    <dependency>profile-plone.app.dexterity:default</dependency>
  </dependencies>
</metadata>
"""
    with open(os.path.join(target_path, "metadata.xml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:upgrade_step",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "plone.version": "5.1",
            "package_folder": package_path,
            "upgrade_step_title": "Add cool index and reindex it",
            "upgrade_step_dest_version": 1005,
            "upgrade_step_description": "We add an index and reindex it with existing content.",
        },
    )
    assert configurator
    upgrade_step._write_dest_version(configurator)
    assert upgrade_step._read_source_version(configurator) == 1005



# -*- coding: utf-8 -*-
from colorama import Fore
from colorama import Style
from datetime import date
from lxml import etree
from mrbob import hooks
from mrbob.bobexceptions import MrBobError
from mrbob.bobexceptions import SkipQuestion
from mrbob.bobexceptions import ValidationError
from mrbob.rendering import jinja2_env
from six.moves import input

import case_conversion as cc
import codecs
import keyword
import os
import six
import string
import subprocess
import sys


try:
    from ConfigParser import ConfigParser
except ImportError:
    from configparser import ConfigParser


ZCML_NAMESPACES = {
    "browser": "http://namespaces.zope.org/browser",
    "plone": "http://namespaces.plone.org/plone",
}


CONTENT_TYPE_INTERFACES = {
    "*": "*",
    "Document": "plone.app.contenttypes.interfaces.IDocument",
    "Folder": "plone.app.contenttypes.interfaces.IFolder",
    "Collection": "plone.app.contenttypes.interfaces.ICollection",
    "Event": "plone.app.contenttypes.interfaces.IEvent",
    "News Items": "plone.app.contenttypes.interfaces.INewsItem",
}


def to_boolean(value):
    if not value:
        return False
    if isinstance(value, six.string_types):
        value = hooks.to_boolean(None, None, value)
    else:
        value = bool(value)
    return value


# this is a custom filter which we can use in jinja template.
# see https://github.com/plone/bobtemplates.plone/issues/292
jinja2_env.filters["to_boolean"] = to_boolean


def git_support_enabled(configurator, question):
    disabled = configurator.variables.get("package.git.disabled", "False")
    if hooks.to_boolean(None, None, disabled):
        echo("GIT support disabled!!!")
        raise SkipQuestion("GIT support is disabled, skip question!")


def echo(msg, msg_type=None):
    if not isinstance(msg, six.string_types):
        msg = str(msg)
    if msg_type == "warning":
        colored_msg = Fore.YELLOW + msg + Style.RESET_ALL
    if msg_type == "error":
        colored_msg = Fore.RED + msg + Style.RESET_ALL
    if msg_type == "info":
        colored_msg = Fore.GREEN + Style.DIM + msg + Style.RESET_ALL
    if not msg_type:
        colored_msg = msg + Style.RESET_ALL
    print(colored_msg)


class BobConfig(object):
    def __init__(self):
        self.version = None
        self.git_init = None
        self.template = None


def git_support(configurator):
    """check if GIT support is disabled/enabled"""
    git_support = True
    disabled = configurator.variables.get("package.git.disabled", "False")
    if hooks.to_boolean(None, None, disabled):
        echo("GIT support disabled!")
        git_support = False
    return git_support


def git_init(configurator):
    if not git_support(configurator):
        return
    git_init_flag = configurator.variables.get("package.git.init", "False")
    if not hooks.to_boolean(None, None, str(git_init_flag)):
        echo("git init is disabled!")
        return
    params = ["git", "init"]
    echo("RUN: {0}".format(" ".join(params)), "info")
    try:
        result = subprocess.check_output(params, cwd=configurator.target_directory)
    except subprocess.CalledProcessError as e:
        echo(e.output, "warning")
    else:
        if result:
            echo(result, "info")
    return True


def git_commit(configurator, msg):
    if not git_support(configurator):
        return
    non_interactive = configurator.bobconfig.get("non_interactive")
    working_dir = (
        configurator.variables.get("package.root_folder")
        or configurator.target_directory
    )
    params1 = ["git", "add", "."]
    params2 = ["git", "commit", "-m", '"{0}"'.format(msg)]
    git_autocommit = None
    run_git_commit = True
    autocommit_flag = configurator.variables.get("package.git.autocommit", "False")
    if hooks.to_boolean(None, None, autocommit_flag):
        git_autocommit = True
    if not non_interactive and not git_autocommit:
        echo(
            "Should we run?:\n{0}\n{1}\nin: {2}".format(
                " ".join(params1), " ".join(params2), working_dir
            ),
            "info",
        )
        run_git_commit = (input("[y]/n: ") or "y").lower() == "y"

    if not run_git_commit and not git_autocommit:
        echo("Skip git commit!", "warning")
        return

    echo("RUN: {0}".format(" ".join(params1)), "info")
    try:
        result1 = subprocess.check_output(params1, cwd=working_dir)
    except subprocess.CalledProcessError as e:
        echo(e.output, "warning")
    else:
        if result1:
            echo(result1, "info")

    echo("RUN: {0}".format(" ".join(params2)), "info")
    try:
        result2 = subprocess.check_output(params2, cwd=working_dir)
    except subprocess.CalledProcessError as e:
        echo(e.output, "warning")
    else:
        echo(result2, "info")


def git_clean_state_check(configurator, question):
    if not git_support(configurator):
        return
    params = ["git", "status", "--porcelain", "--ignore-submodules"]
    echo("\nRUN: {0}".format(" ".join(params)), "info")
    try:
        result = subprocess.check_output(params, cwd=configurator.target_directory)
    except subprocess.CalledProcessError as e:
        echo(e.output, "error")
    else:
        if not result:
            echo("Git state is clean.\n", "info")
            raise SkipQuestion("Git state is clean, so we skip this question.")
        echo(
            "git status result:\n----------------------------\n{0}".format(result),
            "warning",
        )


def check_klass_name(configurator, question, answer):
    if keyword.iskeyword(answer):
        raise ValidationError(
            "{key} is a reserved Python keyword".format(key=answer)
        )  # NOQA: E501
    if not answer.isidentifier():
        raise ValidationError(
            "{key} is not a valid class identifier".format(key=answer)
        )  # NOQA: E501
    return answer


def check_method_name(configurator, question, answer):
    if keyword.iskeyword(answer):
        raise ValidationError(
            "{key} is a reserved Python keyword".format(key=answer)
        )  # NOQA: E501
    if not answer.isidentifier():
        raise ValidationError(
            "{key} is not a valid method identifier".format(key=answer)
        )  # NOQA: E501
    return answer


def read_bobtemplates_ini(configurator):
    bob_config = BobConfig()
    config = ConfigParser()
    path = configurator.target_directory + "/bobtemplate.cfg"
    config.read(path)
    if not config.sections():
        return
    bob_config.version = config.get("main", "version")
    bob_config.git_init = None
    if config.has_option("main", "git_init"):
        bob_config.git_init = config.get("main", "git_init")
    return bob_config


def set_global_vars(configurator):
    bob_config = read_bobtemplates_ini(configurator)
    configurator.variables["year"] = date.today().year
    version = configurator.variables.get("plone.version")
    if not version and bob_config:
        print(">>> reading Plone version from bobtemplate.cfg")
        version = bob_config.version
    configurator.variables["plone.version"] = version
    set_plone_version_variables(configurator)


def set_plone_version_variables(configurator, answer=None):
    version = configurator.variables.get("plone.version", answer)
    if not version:
        return
    if "plone.is_plone5" not in configurator.variables:
        # Find out if it is supposed to be Plone 5.0 or higher
        configurator.variables["plone.is_plone5"] = version.startswith("5")
    if "plone.is_plone51" not in configurator.variables:
        # Find out if it is supposed to be Plone 5.1
        configurator.variables["plone.is_plone51"] = version.startswith("5.1")
    if "plone.is_plone52" not in configurator.variables:
        # Find out if it is supposed to be Plone 5.2
        configurator.variables["plone.is_plone52"] = version.startswith("5.2")
    if "plone.minor_version" not in configurator.variables:
        # extract minor version (4.3)
        # (according to https://plone.org/support/version-support-policy)
        # this is used for the trove classifier in setup.py of the product
        configurator.variables["plone.minor_version"] = ".".join(version.split(".")[:2])


def get_git_info(value):
    """Try to get information from the git-config."""
    gitargs = [b"git", b"config", b"--get"]
    try:
        result = subprocess.check_output(gitargs + [value]).strip()
        if six.PY3 and isinstance(result, six.binary_type):
            result = result.decode("utf8")
        return result
    except (OSError, subprocess.CalledProcessError):
        return "FakeGitUserOrEmail"


def validate_packagename(configurator):
    """Find out if the name target-dir entered when invoking the command can be
    a valid python-package."""
    package_dir = os.path.basename(configurator.target_directory)
    fail = False

    allowed = set(string.ascii_letters + string.digits + ".-_")
    if not set(package_dir).issubset(allowed):
        fail = True

    if package_dir.startswith(".") or package_dir.endswith("."):
        fail = True

    for namespace in package_dir.replace("-", "_").split("."):
        if keyword.iskeyword(namespace) or not namespace.isidentifier():
            fail = True

    if fail:
        msg = (
            "Error: '{0}' is not a valid packagename.\n"
            "Please use a valid name (like collective.myaddon or "
            "plone.app.myaddon)".format(package_dir)
        )
        sys.exit(msg)


def post_plone_version(configurator, question, answer):
    """Find out if it is supposed to be Plone 5."""
    set_plone_version_variables(configurator, answer)
    return answer


def pre_username(configurator, question):
    """Get email from git and validate package name."""
    # validate_packagename should be run before asking the first question.
    validate_packagename(configurator)

    default = get_git_info("user.name")
    if default and question:
        question.default = default


def pre_email(configurator, question):
    """Get email from git."""
    default = get_git_info("user.email")
    if default and question:
        question.default = default


def is_string_in_file(configurator, file_path, match_str):
    """Simple check if a given string is in a file.

    You can use this before adding new lines with update_file.

    """
    with open(file_path, "r+") as xml_file:
        contents = xml_file.readlines()
    for index, line in enumerate(contents):
        if match_str in line:
            return True
    return False


def make_path(*args):
    """generate path string."""
    return os.sep.join(args)


def update_configure_zcml(
    configurator,
    path,
    file_name=None,
    example_file_name=None,
    match_xpath=None,
    match_str=None,
    insert_str=None,
):
    if path[-1] != "/":
        path += "/"
    file_path = os.path.join(path, file_name)
    if example_file_name:
        example_file_path = os.path.join(path, example_file_name)
        file_list = os.listdir(os.path.dirname(path))
        if file_name not in file_list:
            print("rename example zcml file")
            os.rename(example_file_path, file_path)
    namespaces = {
        "zope": "http://namespaces.zope.org/zope",
        "gs": "http://namespaces.zope.org/genericsetup",
        "i18n": "http://namespaces.zope.org/i18n",
        "plone": "http://namespaces.plone.org/plone",
    }
    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        if len(tree.xpath(match_xpath, namespaces=namespaces)):
            print("{0} already in {1}, skip adding!".format(insert_str, file_path))
            return
    update_file(configurator, file_path, match_str, insert_str)


def update_file(configurator, file_path, match_str, insert_str):
    """Insert insert_str into given file, by match_str."""
    changed = False
    with codecs.open(file_path, "r+", encoding="utf-8") as xml_file:
        contents = xml_file.readlines()
        if match_str in contents[-1]:  # Handle last line, prev. IndexError
            contents.append(insert_str)
            changed = True
        else:
            for index, line in enumerate(contents):
                if match_str in line and insert_str not in contents[index + 1]:
                    contents.insert(index + 1, insert_str)
                    changed = True
                    break
        xml_file.seek(0)
        xml_file.writelines(contents)

    if not changed:
        print(
            "WARNING: We couldn't find the match_str, "  # NOQA
            "skip inserting into {0}:\n".format(file_path)  # NOQA
        )


def _get_package_root_folder(configurator):
    file_name = "setup.py"
    root_folder = None
    os.chdir(configurator.target_directory)
    cur_dir = os.getcwd()
    while True:
        files = os.listdir(cur_dir)
        parent_dir = os.path.dirname(cur_dir)
        if file_name in files:
            root_folder = cur_dir
            break
        else:
            if cur_dir == parent_dir:
                break
            cur_dir = parent_dir
    return root_folder


def check_root_folder(configurator, question):
    """Check if we are in a package.

    Should be called in first question pre hook.

    """
    root_folder = _get_package_root_folder(configurator)
    if not root_folder:
        raise ValidationError(
            "\n\nNo setup.py found in path!\n"
            "Please run this subtemplate inside an existing package,\n"
            "in the package dir, where the actual code is!\n"
            "In the package collective.dx it's in collective.dx/collective/dx"
            "\n"
        )


def dottedname_to_path(dottedname):
    path = "/".join(dottedname.split("."))
    return path


def base_prepare_renderer(configurator):
    """generic rendering before template specific rendering."""
    configurator.variables["package.root_folder"] = _get_package_root_folder(
        configurator
    )
    if not configurator.variables["package.root_folder"]:
        raise MrBobError("No setup.py found in path!\n")
    configurator.variables["package.dottedname"] = configurator.variables[
        "package.root_folder"
    ].split(os.path.sep)[-1]
    configurator.variables["package.namespace"] = configurator.variables[
        "package.dottedname"
    ].split(".")[0]
    configurator.variables["package.name"] = configurator.variables[
        "package.dottedname"
    ].split(".")[-1]
    # package.uppercasename = 'COLLECTIVE_FOO_SOMETHING'
    configurator.variables["package.uppercasename"] = (
        configurator.variables["package.dottedname"].replace(".", "_").upper()
    )

    package_subpath = dottedname_to_path(configurator.variables["package.dottedname"])
    configurator.variables["package_folder_rel_path"] = "/src/" + package_subpath
    configurator.variables["package_folder"] = (
        configurator.variables["package.root_folder"]
        + configurator.variables["package_folder_rel_path"]
    )
    configurator.target_directory = configurator.variables["package.root_folder"]
    camelcasename = (
        configurator.variables["package.dottedname"]
        .replace(".", " ")
        .title()
        .replace(" ", "")
        .replace("_", "")
    )
    browserlayer = "{0}Layer".format(camelcasename)

    # package.browserlayer = 'CollectiveFooSomethingLayer'
    configurator.variables["package.browserlayer"] = browserlayer
    set_global_vars(configurator)
    return configurator


def remove_unwanted_files(file_paths):
    for file_path in file_paths:
        if not os.path.isfile(file_path):
            continue
        os.remove(file_path)


def subtemplate_warning(configurator, question):
    """Show a warning to the user before using subtemplates!"""
    print(
        """
    ### WARNING ###

    This is a subtemplate, it might override existing files without warnings!
    Please use a version control system like GIT with a clean state,
    to track changes, before using this subtemplate!

    """,
    )


def subtemplate_warning_post_question(configurator, question, answer):
    if answer.lower() != "y":
        print("Abort!")
        sys.exit(0)
    return answer


def get_normalized_dxtypename(name):
    normalized_name = cc.snakecase(get_normalized_ftiname(name))
    return normalized_name


def get_normalized_classname(name):
    normalized_name = cc.pascalcase(name)
    return normalized_name


def get_normalized_ftiname(name):
    normalized_name = name.replace(" ", "_")
    return normalized_name


def get_normalized_themename(name):
    name = name.replace("(", "")
    name = name.replace(")", "")
    name = name.replace("-", "")
    name = name.replace(" ", "_")
    normalized_name = cc.dashcase(name)
    return normalized_name



# -*- coding: utf-8 -*-
from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import echo
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import git_init
from mrbob.bobexceptions import ValidationError

import os
import re


def pre_render(configurator):
    """Prepare configuration variables."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "mockup_pattern"

    bundle_js_path = os.path.join(
        configurator.variables["package.root_folder"], "resources/bundle.js"
    )
    configurator.variables["original_imports"] = ""
    if os.path.exists(bundle_js_path):
        with open(bundle_js_path, "r") as bundle_js:
            configurator.variables["original_imports"] = "\n".join(
                [
                    line.strip()
                    for line in bundle_js.readlines()
                    if "import" in line and "patternslib" not in line
                ]
            )

    configurator.variables["original_body"] = value_from_template(
        configurator.variables["package.root_folder"],
        "resources/index.html",
        r"\<body\>(.*)\<\/body\>",
    )

    configurator.variables["original_browser_configure"] = value_from_template(
        configurator.variables["package_folder"],
        "browser/configure.zcml",
        r"(\<configure.*)\<\/configure\>",
    )

    configurator.variables["original_browser_template"] = (
        value_from_template(
            configurator.variables["package_folder"],
            "browser/pattern-demo.pt",
            r"<metal:block define-macro=\"content-core\"\>(.*)\<\/metal:block\>",
        )
        or ""
    )


def value_from_template(root_folder, relative_path, regex):
    path = os.path.join(root_folder, relative_path)
    if os.path.exists(path):
        with open(path, "r") as file:
            # Read the HTML file and extract the body
            contents = file.read()
            re_pattern = re.compile(regex, flags=re.DOTALL)
            return "".join(re_pattern.findall(contents))


def post_render(configurator):
    """Post render script."""
    root_folder = configurator.variables["package.root_folder"]
    pattern_name = configurator.variables["pattern.name"]
    package_name = configurator.variables["package.name"]
    package_folder_rel_path = configurator.variables["package_folder_rel_path"]

    git_init_status = git_init(configurator)
    if git_init_status:
        git_commit(
            configurator,
            f"Add pattern: {pattern_name}",
        )
    echo(
        f"""
Your pattern was added here: {root_folder}/resources/pat-{pattern_name}

Run "npx yarn install" to get the dependencies.
Run "npx yarn start" to start the development server for developing your pattern.
Run "npx yarn watch" to re-build the pattern into the Plone environment on changes.
Run "npx yarn run build" to compile the javascript bundle for production.

Note: You need to build the bundle before you can use the pattern in Plone.

Note: You might want to edit and fix the following file - there might be multiple
      "{package_name}-pattern-demo" view configurations in it:

.{package_folder_rel_path}/browser/configure.zcml

There is a demo view for your pattern at:

http://localhost:8080/Plone/@@{package_name}-pattern-demo

""",
        "info",
    )


def post_pattern_name(configurator, question, answer):
    """Check name."""
    regex = r"^\w+[a-zA-Z0-9\.\-_]*$"
    if not re.match(regex, answer):
        msg = f"""Error: "{answer}" is not a valid pattern name.
Please use a valid name (like "Autoselect" or "show-hide")!
At beginning or end only letters|digits are allowed.
Inside the name ".-_" are also allowed.
No accents or umlauts."""
        raise ValidationError(msg)
    return answer



# -*- coding: utf-8 -*-
from .base import init_package_base_structure
from bobtemplates.plone import base
from bobtemplates.plone import mockup_pattern
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator

import os
import pytest


def test_pre_render(tmpdir):
    package_root = tmpdir.strpath + "/collective.testpattern"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:mockup_pattern",
        target_directory=package_path,
        variables={
            "pattern.name": "test-pattern",
        },
    )
    mockup_pattern.pre_render(configurator)

    assert configurator.variables["template_id"] == "mockup_pattern"
    assert configurator.variables["pattern.name"] == "test-pattern"
    assert configurator.target_directory.endswith("collective.testpattern")


def test_post_render(tmpdir):
    package_root = tmpdir.strpath + "/collective.testpattern"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:mockup_pattern",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={
            "plone.version": "6.0",
            "pattern.name": "test-pattern",
        },
    )
    configurator = Configurator(
        template="bobtemplates.plone:theme_barceloneta",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={"plone.version": "5.1", "theme.name": "My Theme"},
    )

    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    configurator.render()  # pre/render/post


def test_post_pattern_name(tmpdir):
    """Verifies that pattern names are checked for validity."""
    target_path = tmpdir.strpath + "/collective.testpattern"
    configurator = Configurator(
        template="bobtemplates.plone:mockup_pattern", target_directory=target_path
    )

    mockup_pattern.post_pattern_name(configurator, None, "test-pattern")
    with pytest.raises(ValidationError):
        mockup_pattern.post_pattern_name(configurator, None, "test.$SPAM")



# -*- coding: utf-8 -*-

"""Test view generation."""

from bobtemplates.plone import base
from bobtemplates.plone import portlet
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator

import os
import pytest


def test_pre_renderer():
    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory=".",
        variables={
            "portlet_name": "My nice portlet, with umlauts: öÖÖÖÖ".encode(
                "utf8",
            ),
        },
    )
    portlet.prepare_renderer(configurator)
    expt = "my_nice_portlet_with_umlauts_ooooo"
    assert configurator.variables["portlet_name_normalized"] == expt


def test_update_configure_zcml_with_changes(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.demo"
    package_path = target_path + "/src/collective/demo"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.demo">

  <i18n:registerTranslations directory="locales" />

  <!--<includeDependencies package="." />-->

  <include package=".portlets" />

  <include package=".browser" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory="collective.demo",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
        },
    )
    portlet._update_configure_zcml(configurator)

    with open(os.path.join(package_path + "/configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_update_configure_zcml_without_changes(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.demo"
    package_path = target_path + "/src/collective/demo"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.demo">

  <i18n:registerTranslations directory="locales" />

  <!--<includeDependencies package="." />-->

  <include package=".browser" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory="collective.demo",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
        },
    )
    portlet._update_configure_zcml(configurator)

    complete_template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.demo">

  <i18n:registerTranslations directory="locales" />

  <!--<includeDependencies package="." />-->

  <include package=".portlets" />

  <include package=".browser" />

</configure>
"""

    with open(os.path.join(package_path + "/configure.zcml"), "r") as f:
        content = f.read()
        if content != complete_template:
            pytest.raises(ValidationError)


def test_update_portlets_configure_zcml(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    portlets_path = package_path + "/portlets/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(portlets_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.sample">

  '-*- extra stuff goes here -*-'

  <plone:portlet
    name="collective.sample.portlets.MyWeather"
    interface=".my_weather.IMyWeatherPortlet"
    assignment=".my_weather.Assignment"
    renderer=".my_weather.Renderer"
    addview=".my_weather.AddForm"
    editview=".my_weather.EditForm" />

</configure>
"""
    with open(os.path.join(portlets_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "portlet_name": "MyWeather",
            "portlet_name_normalized": "my_weather",
            "portlet_configuration_name": "collective.sample.portlets.MyWeather",  # NOQA: E501
            "data_provider_class_name": "IMyWeatherPortlet",
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
        },
    )
    portlet._update_portlets_configure_zcml(configurator)

    with open(os.path.join(portlets_path + "configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_delete_unnecessary_files(tmpdir):
    """Test to remove unwanted file."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    portlets_path = package_path + "/portlets/"
    profile_path = package_path + "/profiles/default/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(portlets_path)
    os.makedirs(profile_path)
    file_name = "configure.zcml.example"
    portlets_file_path = portlets_path + file_name
    open(portlets_file_path, "a").close()
    file_name = "portlets.xml.example"
    xml_file_path = profile_path + file_name
    open(xml_file_path, "a").close()
    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
        },
    )
    portlet._delete_unnecessary_files(configurator)
    if os.path.isfile(portlets_file_path):
        pytest.raises(ValidationError)
    if os.path.isfile(xml_file_path):
        pytest.raises(ValidationError)


def test_update_portlets_xml(tmpdir):
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    profile_path = package_path + "/profiles/default/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profile_path)
    template = """<?xml version="1.0"?>
<portlets
    xmlns:i18n="http://xml.zope.org/namespaces/i18n"
    i18n:domain="plone">
  <!-- Extra portlets here  -->


</portlets>"""
    with open(os.path.join(profile_path + "portlets.xml.example"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "portlet_name": "My Weather",
            "portlet_name_normalized": "my_weather",
            "portlet_configuration_name": "collective.sample.portlets.MyWeather",  # NOQA: E501
            "package_folder": package_path,
        },
    )
    portlet._update_portlets_xml(configurator)

    template = """<?xml version="1.0"?>
<portlets
    xmlns:i18n="http://xml.zope.org/namespaces/i18n"
    i18n:domain="plone">
  <!-- Extra portlets here  -->

  <portlet
    addview="collective.sample.portlets.MyWeather"
    title="My Weather"
    description="A portlet which can render weather of the given place."
    i18n:attributes="title title_my_weather;
                     description description_my_weather">

    <!-- This will enable the portlet for right column,
    left column and the footer too.
    -->
    <for interface="plone.app.portlets.interfaces.IColumn" />

    <!--
    This will enable the portlet in the dashboard.
    -->
    <!--<for interface="plone.app.portlets.interfaces.IDashboard" />-->

  </portlet>


</portlets>"""

    with open(os.path.join(profile_path + "portlets.xml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_update_portlets_xml_with_changes(tmpdir):
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    profile_path = package_path + "/profiles/default/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profile_path)
    template = """<?xml version="1.0"?>
<portlets
    xmlns:i18n="http://xml.zope.org/namespaces/i18n"
    i18n:domain="plone">
  <!-- Extra portlets here  -->

  <portlet
    addview="collective.sample.portlets.MyWeather"
    title="My Weather"
    description="A portlet which can render weather of the given place."
    i18n:attributes="title title_my_weather;
                     description description_my_weather">

    <!-- This will enable the portlet for right column,
    left column and the footer too.
    -->
    <for interface="plone.app.portlets.interfaces.IColumn" />

    <!--
    This will enable the portlet in the dashboard.
    -->
    <!--<for interface="plone.app.portlets.interfaces.IDashboard" />-->

  </portlet>

</portlets>"""

    with open(os.path.join(profile_path + "portlets.xml.example"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:portlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "portlet_name": "My Weather",
            "portlet_name_normalized": "my_weather",
            "portlet_configuration_name": "collective.sample.portlets.MyWeather",  # NOQA: E501
            "package_folder": package_path,
        },
    )
    portlet._update_portlets_xml(configurator)

    with open(os.path.join(profile_path + "portlets.xml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_post_renderer(tmpdir):
    """Test post rendering."""
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    portlets_path = package_path + "/portlets"
    profile_path = package_path + "/profiles/default/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(portlets_path)
    os.makedirs(profile_path)

    template = """# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from collective.todo import _
from plone import schema
from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import field
from zope.component import getMultiAdapter
from zope.interface import implementer

import json
import urllib
import urllib2


class IMyWeatherPortlet(IPortletDataProvider):
    place_str = schema.TextLine(
        title=_(u'Name of your place with country code'),
        description=_(u'City name along with country code i.e Delhi,IN'),  # NOQA: E501
        required=True,
        default=u'delhi,in'
    )


@implementer(IMyWeatherPortlet)
class Assignment(base.Assignment):
    schema = IMyWeatherPortlet

    def __init__(self, place_str='delhi,in'):
        self.place_str = place_str.lower()

    @property
    def title(self):
        return _(u'Weather of the place')


class AddForm(base.AddForm):
    schema = IMyWeatherPortlet
    form_fields = field.Fields(IMyWeatherPortlet)
    label = _(u'Add Place weather')
    description = _(u'This portlet displays weather of the place.')

    def create(self, data):
        return Assignment(
            place_str=data.get('place_str', 'delhi,in'),
        )


class EditForm(base.EditForm):
    schema = IMyWeatherPortlet
    form_fields = field.Fields(IMyWeatherPortlet)
    label = _(u'Edit Place weather')
    description = _(u'This portlet displays weather of the place.')


class Renderer(base.Renderer):
    schema = IMyWeatherPortlet
    _template = ViewPageTemplateFile('my_weather.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)
        context = aq_inner(self.context)
        portal_state = getMultiAdapter(
            (context, self.request),
            name=u'plone_portal_state'
        )
        self.anonymous = portal_state.anonymous()

    def render(self):
        return self._template()

    @property
    def available(self):
        \"""Show the portlet only if there are one or more elements.\"""
        return not self.anonymous and self._data()

    def weather_report(self):
        self.result = self._data()
        return self.result['description']

    def get_humidity(self):
        return self.result['humidity']

    def get_pressure(self):
        return self.result['pressure']
"""
    with open(os.path.join(portlets_path + "/my_weather.py"), "w") as f:
        f.write(template)

    template = """<div class="weather">
    <h3>Weather Report</h3>
    ${view/weather_report}
    <br>
    Pressure: ${view/get_pressure}
    <br>
    Humidity: ${view/get_humidity}
    <br>
</div>
"""
    with open(os.path.join(portlets_path + "/my_weather.pt"), "w") as f:
        f.write(template)
    template = """<configure
  xmlns="http://namespaces.zope.org/zope"
  xmlns:browser="http://namespaces.zope.org/browser"
  xmlns:plone="http://namespaces.plone.org/plone">

  <include package="plone.app.portlets" />

  '-*- extra stuff goes here -*-'

  <plone:portlet
    name="collective.todo.portlets.MyWeather"
    interface=".my_weather.IMyWeatherPortlet"
    assignment=".my_weather.Assignment"
    renderer=".my_weather.Renderer"
    addview=".my_weather.AddForm"
    editview=".my_weather.EditForm" />

</configure>"""
    with open(os.path.join(portlets_path + "/configure.zcml"), "w") as f:
        f.write(template)

    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.todo">


  <i18n:registerTranslations directory="locales" />

  <!--
    Be careful if you use general includeDependencies, it can have sideffects!
    Better import explicite packages or configurations ;)
  -->
  <!--<includeDependencies package="." />-->

  <include package=".portlets" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)

    template = """
        dummy
        '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    template = """<?xml version="1.0"?>
<portlets
    xmlns:i18n="http://xml.zope.org/namespaces/i18n"
    i18n:domain="plone">
  <!-- Extra portlets here  -->

  <portlet
    addview="collective.todo.portlets.MyWeather"
    title="My Weather"
    description="A portlet which can render weather of the given place."
    i18n:attributes="title title_my_weather;
                     description description_my_weather">

    <!-- This will enable the portlet for right column,
    left column and the footer too.
    -->
    <for interface="plone.app.portlets.interfaces.IColumn" />

    <!--
    This will enable the portlet in the dashboard.
    -->
    <!--<for interface="plone.app.portlets.interfaces.IDashboard" />-->

  </portlet>


</portlets>
"""
    with open(os.path.join(profile_path + "/portlets.xml.example"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "portlet_name": "My Weather",
            "plone.version": "5.1",
        },
    )
    os.chdir(package_path)
    base.set_global_vars(configurator)
    portlet.prepare_renderer(configurator)
    portlet.post_renderer(configurator)



# -*- coding: utf-8 -*-

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import echo
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import update_file
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort
from lxml import etree
from mrbob.bobexceptions import ValidationError

import case_conversion as cc
import re


def check_name(configurator, question, answer):
    if not re.match("^[a-z]+-+[a-z0-9]+(-+[a-z0-9]+)*$", answer):
        raise ValidationError(
            "{key} is not a valid custom-element identifier. Please try something like this 'my-element'".format(
                key=answer
            )
        )  # NOQA: E501
    return answer


def _update_configure_zcml(configurator):
    file_name = "configure.zcml"
    file_path = configurator.variables["package_folder"] + "/" + file_name
    namespaces = {"plone": "http://namespaces.plone.org/plone"}

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        xpath = "./plone:static[@name='{}.svelte']".format(
            configurator.variables["package.dottedname"]
        )
        if len(tree_root.xpath(xpath, namespaces=namespaces)):
            print(
                "{name}.svelte already in configure.zcml, skip adding!".format(
                    name=configurator.variables["package.dottedname"]
                )
            )
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <plone:static
      directory="svelte_apps"
      type="plone"
      name="{0}.svelte"
      />

""".format(
        configurator.variables["package.dottedname"]
    )
    update_file(configurator, file_path, match_str, insert_str)


def pre_renderer(configurator):
    """Pre rendering."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "svelte_app"
    name = configurator.variables["svelte_app_name"].strip("_")
    configurator.variables["svelte_app_file_name"] = cc.snakecase(name)
    configurator.variables["svelte_app_name_dashed"] = cc.dashcase(name)


def post_renderer(configurator):
    """Post rendering."""
    _update_configure_zcml(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_commit(
        configurator,
        "Add Svelte app: in svelte_apps/{0}".format(
            configurator.variables["svelte_app_name"],
        ),
    )
    echo(
        "===================================================\n"
        "=> Sucessfully added: {0} in svelte_apps/{1} \n"
        "=> you might want to go into the dir and run:\n"
        "$ yarn\n"
        "and then:"
        "$ yarn dev\n".format(
            configurator.variables["template_id"],
            configurator.variables["svelte_app_name"],
        ),
        "info",
    )



# -*- coding: utf-8 -*-

from bobtemplates.plone import base
from bobtemplates.plone import restapi_service
from mrbob.configurator import Configurator

import os


def test_pre_renderer(tmpdir):
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:restapi_service",
        target_directory=target_path,
        variables={
            "service_class_name": "SomeRelatedThings",
            "service_name": "some-related-things",
            "package_folder": package_path,
        },
    )
    restapi_service.pre_renderer(configurator)


def test_post_renderer(tmpdir):
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    profiles_path = package_path + "/profiles/default"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(profiles_path)

    template = """<?xml version="1.0" encoding="UTF-8"?>
<metadata>
  <version>1000</version>
  <dependencies>

  </dependencies>
</metadata>
"""
    with open(os.path.join(profiles_path + "/metadata.xml"), "w") as f:
        f.write(template)

    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">

    <!-- -*- extra stuff goes here -*- -->

    </configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:restapi_service",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
            "plone.version": "5.1",
            "service_class_name": "SomeRelatedThings",
            "service_name": "some-related-things",
        },
    )
    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    restapi_service.pre_renderer(configurator)
    configurator.render()
    restapi_service.post_renderer(configurator)


def test_remove_unwanted_files(tmpdir):
    files_to_remove = [
        "/api/configure.zcml.example",
        "/api/services/configure.zcml.example",
    ]
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    os.makedirs(package_path + "/api/services/")
    configurator = Configurator(
        template="bobtemplates.plone:restapi_service",
        target_directory=tmpdir.strpath,
        variables={
            "package_folder": package_path,
        },
    )
    for file_to_remove in files_to_remove:
        with open(
            os.path.join(
                package_path + file_to_remove,
            ),
            "w",
        ) as f:
            f.write("dummy")
    restapi_service._remove_unwanted_files(configurator)

    for file_to_remove in files_to_remove:
        assert not os.path.isfile(
            os.path.join(package_path + file_to_remove),
        )


def test_update_api_configure_zcml(tmpdir):
    """ """
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    os.makedirs(package_path + "/api/")

    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">

    <!-- -*- extra stuff goes here -*- -->

    </configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    with open(os.path.join(package_path + "/api/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:restapi_service",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
            "plone.version": "5.1",
            "service_class_name": "SomeRelatedThings",
            "service_name": "some-related-things",
        },
    )
    restapi_service._update_api_configure_zcml(configurator)

    with open(
        os.path.join(
            package_path + "/api/configure.zcml",
        ),
        "r",
    ) as f:
        content = f.read()
        assert content != template, "configure.zcml was not updated!"


def test_update_services_configure_zcml(tmpdir):
    """ """
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    os.makedirs(package_path + "/api/services/")

    template = """
[main]
version=5.1
"""
    with open(os.path.join(target_path + "/bobtemplate.cfg"), "w") as f:
        f.write(template)

    template = """
    dummy
    '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    template = """
    <configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone">

    <!-- -*- extra stuff goes here -*- -->

    </configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    with open(os.path.join(package_path + "/api/configure.zcml"), "w") as f:
        f.write(template)
    with open(
        os.path.join(package_path + "/api/services/configure.zcml"),
        "w",
    ) as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:restapi_service",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
            "plone.version": "5.1",
            "service_class_name": "SomeRelatedThings",
            "service_class_name_normalized": "some_related_things",
            "service_name": "some-related-things",
        },
    )
    restapi_service._update_services_configure_zcml(configurator)

    with open(
        os.path.join(
            package_path + "/api/services/configure.zcml",
        ),
        "r",
    ) as f:
        content = f.read()
        assert content != template, "configure.zcml was not updated!"



# -*- coding: utf-8 -*-
"""Generate form."""

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import CONTENT_TYPE_INTERFACES
from bobtemplates.plone.base import echo
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import update_file
from bobtemplates.plone.base import ZCML_NAMESPACES
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort
from lxml import etree

import case_conversion as cc
import os
import six


# from mrbob.bobexceptions import SkipQuestion, ValidationError


def get_form_name_from_python_class(configurator, question):
    """Generate form default name from python class"""
    form_class_name = configurator.variables["form_python_class_name"]
    form_generated_name = cc.snakecase(form_class_name).replace("_", "-")  # NOQA: E501
    question.default = form_generated_name


def get_form_configuration(configurator):
    """return a dict with form configuration used for registration in zcml"""
    config = dict()
    config["name"] = configurator.variables["form_name"]
    # get Interface by content type or use the string it self as interface
    config["for"] = "{0}".format(
        CONTENT_TYPE_INTERFACES.get(
            configurator.variables["form_register_for"],
            configurator.variables["form_register_for"],
        )
    )
    config["class"] = ".{0}.{1}".format(
        configurator.variables["form_python_file_name"],
        configurator.variables["form_python_class_name"],
    )
    # if configurator.variables["form_permission"]:
    config["permission"] = "cmf.ManagePortal"
    return config


def _update_forms_configure_zcml(configurator):
    file_name = "configure.zcml"
    directory_path = configurator.variables["package_folder"] + "/forms/"
    file_path = directory_path + file_name
    configure_example_file_path = (
        configurator.variables["package_folder"] + "/forms/configure.zcml.example"
    )
    file_list = os.listdir(os.path.dirname(directory_path))
    if file_name not in file_list:
        os.rename(configure_example_file_path, file_path)

    match_str = "-*- extra stuff goes here -*-"

    form_config = get_form_configuration(configurator)
    insert_str = """
  <browser:page
    name="{0}"
    for="{1}"
""".format(
        form_config["name"],
        form_config["for"],
    )
    if "class" in form_config:
        insert_str += '    class="{0}"\n'.format(form_config["class"])
    # if "template" in form_config:
    #     insert_str += '    template="{0}"\n'.format(form_config["template"])
    if "permission" in form_config:
        insert_str += '    permission="{0}"\n'.format(form_config["permission"])
    insert_str += '    layer="{0}.interfaces.I{1}"\n'.format(
        configurator.variables["package.dottedname"],
        configurator.variables["package.browserlayer"],
    )
    insert_str += "    />\n"

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        form_xpath = "./browser:page[@name='{0}']".format(
            configurator.variables["form_name"],
        )
        if len(tree_root.xpath(form_xpath, namespaces=ZCML_NAMESPACES)):
            echo(
                "{0} already in configure.zcml, do you really want to add this config?"
                "\n\n{1}\n [y/N]: ".format(
                    configurator.variables["form_name"],
                    insert_str,
                ),
                "info",
            )
            if configurator.bobconfig.get("non_interactive"):
                return
            choice = six.moves.input().lower()
            if choice != "y":
                return

    update_file(configurator, file_path, match_str, insert_str)


def _update_configure_zcml(configurator):
    file_name = "configure.zcml"
    file_path = configurator.variables["package_folder"] + "/" + file_name
    namespaces = "{http://namespaces.zope.org/zope}"

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        form_xpath = "{0}include[@package='.forms']".format(namespaces)
        if len(tree_root.findall(form_xpath)):
            print(
                ".forms already in configure.zcml, skip adding!",
            )
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".forms" />
"""
    update_file(configurator, file_path, match_str, insert_str)


def _delete_unwanted_files(configurator):
    directory_path = configurator.variables["package_folder"] + "/forms/"

    file_name = "configure.zcml.example"
    file_list = os.listdir(os.path.dirname(directory_path))
    if file_name in file_list:
        file_path = directory_path + file_name
        os.remove(file_path)


def prepare_renderer(configurator):
    """Prepare rendering."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "form"
    form_name = configurator.variables["form_name"].strip("_")
    normalized_form_name = cc.snakecase(form_name)
    configurator.variables["form_name_normalized"] = normalized_form_name
    python_class_name = configurator.variables["form_python_class_name"].strip("_")
    configurator.variables["form_python_class_name"] = cc.pascalcase(  # NOQA: E501
        python_class_name,
    )
    form_python_file_name = cc.snakecase(python_class_name)
    configurator.variables["form_python_file_name"] = form_python_file_name
    form_name_from_input = normalized_form_name.replace("_", "-")
    form_name_from_python_class = form_python_file_name.replace("_", "-")
    if form_name_from_input != form_name_from_python_class:
        configurator.variables["form_name"] = form_name_from_input

    configurator.target_directory = configurator.variables["package_folder"]


def post_renderer(configurator):
    """Post rendering."""
    _update_configure_zcml(configurator)
    _update_forms_configure_zcml(configurator)
    _delete_unwanted_files(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_commit(
        configurator,
        "Add form: {0}".format(
            configurator.variables["form_name"],
        ),
    )



# -*- coding: utf-8 -*-

from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import git_init
from bobtemplates.plone.base import make_path
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort

import os
import shutil


def pre_render(configurator):
    """Some variables to make templating easier."""
    # TODO: refacture this to use base_prepare_renderer, like all sub template do
    # get package-name from user-input
    package_dir = os.path.basename(configurator.target_directory)

    namespaces = package_dir.replace("-", "_").split(".")

    configurator.variables["package.namespace"] = ".".join(namespaces[:-1])
    configurator.variables["package.dottedname"] = ".".join(namespaces)
    configurator.variables["package.name"] = namespaces[-1]

    # DISTRIBUTION NAME
    configurator.variables["package.distributionname"] = package_dir

    # package.uppercasename = 'COLLECTIVE_FOO_SOMETHING'
    configurator.variables["package.uppercasename"] = (
        configurator.variables["package.dottedname"].replace(".", "_").upper()
    )

    camelcasename = (
        configurator.variables["package.dottedname"]
        .replace(".", " ")
        .title()
        .replace(" ", "")
        .replace("_", "")
    )
    browserlayer = "{0}Layer".format(camelcasename)

    # package.browserlayer = 'CollectiveFooSomethingLayer'
    configurator.variables["package.browserlayer"] = browserlayer

    # package.longname = 'collectivefoosomething'
    configurator.variables["package.longname"] = camelcasename.lower()

    # jenkins.directories = 'collective/foo/something'
    configurator.variables["jenkins.directories"] = configurator.variables[
        "package.distributionname"
    ].replace(
        ".", "/"
    )  # NOQA: E501

    if namespaces:
        configurator.variables["package.namespace_packages"] = ", ".join(
            "'{0}'".format(".".join(namespaces[:idx]))
            for idx in range(1, len(namespaces))
        )
    else:
        configurator.variables["package.namespace_packages"] = ""


def _cleanup_package(configurator):
    """Cleanup and make nested if needed.

    Transform into a nested package if that was the selected option.
    Remove parts that are not needed depending on the chosen
    configuration.

    """

    start_path = configurator.target_directory

    if configurator.variables["package.namespace"]:
        newpath = make_path(
            start_path,
            "src",
            *configurator.variables["package.namespace"].split("."),
        )
        if not os.path.exists(newpath):
            # create new directory .../src/collective/behavior
            os.makedirs(newpath)
        oldpath = make_path(
            start_path,
            "src",
            configurator.variables["package.namespace"],
        )
        # Event though the target-dir was 'collective.behavior.myaddon' mrbob
        # created a package collective.behavior.myaddon/src/collective/myaddon
        # since the template does not hava a folder for namespace2.
        # Here this package is turned into a nested package
        # collective.behavior.myaddon/src/collective/behavior/myaddon by
        # inserting a folder with the namepsace2 ('behavior') and copying
        # a __init__.py into it.
        if oldpath != newpath:
            # move .../src/collective/myaddon to .../src/collective/behavior
            if not os.path.exists(
                make_path(newpath, configurator.variables["package.name"])
            ):
                shutil.move(
                    make_path(oldpath, configurator.variables["package.name"]), newpath
                )
            init = make_path(oldpath, "__init__.py")
            namespaces = configurator.variables["package.namespace"].split(".")
            for idx in range(len(namespaces)):
                shutil.copy(init, make_path(start_path, "src", *namespaces[: idx + 1]))
            shutil.rmtree(oldpath)


def pre_ask(configurator):
    """ """


def post_render(configurator):
    _cleanup_package(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_init_status = git_init(configurator)
    if git_init_status:
        git_commit(
            configurator,
            "Create addon: {0}".format(
                configurator.variables["package.dottedname"],
            ),
        )



# -*- coding: utf-8 -*-

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import remove_unwanted_files
from bobtemplates.plone.base import update_configure_zcml
from lxml import etree


def _update_package_configure_zcml(configurator):
    path = "{0}".format(
        configurator.variables["package_folder"],
    )
    file_name = "configure.zcml"
    match_xpath = "zope:include[@package='.upgrades']"
    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".upgrades" />
"""
    update_configure_zcml(
        configurator,
        path,
        file_name=file_name,
        match_xpath=match_xpath,
        match_str=match_str,
        insert_str=insert_str,
    )


def _update_upgrades_configure_zcml(configurator):
    path = "{0}/upgrades".format(
        configurator.variables["package_folder"],
    )
    file_name = "configure.zcml"
    example_file_name = "{0}.example".format(file_name)
    zcml_package_name = configurator.variables["upgrade_step_dest_version"]
    match_xpath = "zope:include[@file='{0}.zcml']".format(zcml_package_name)
    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include file="{0}.zcml" />
""".format(
        zcml_package_name
    )
    update_configure_zcml(
        configurator,
        path,
        file_name=file_name,
        example_file_name=example_file_name,
        match_xpath=match_xpath,
        match_str=match_str,
        insert_str=insert_str,
    )


def _remove_unwanted_files(configurator):
    file_paths = []
    rel_file_paths = [
        "/upgrades/configure.zcml.example",
    ]
    base_path = configurator.variables["package_folder"]
    for rel_file_path in rel_file_paths:
        file_paths.append("{0}{1}".format(base_path, rel_file_path))
    remove_unwanted_files(file_paths)


def _read_source_version(configurator):
    base_path = configurator.variables["package_folder"]
    rel_file_path = "/profiles/default/metadata.xml"
    metadata_path = "{0}{1}".format(base_path, rel_file_path)
    with open(metadata_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        match_xpath = "version"
        match_result = tree_root.findall(match_xpath)
        if not match_result:
            raise RuntimeError("source version not found in metadata.xml!")
            return
        return int(match_result[0].text)


def _write_dest_version(configurator):
    """Add plone.app.dexterity dependency metadata.xml in Generic Setup profiles."""  # NOQA: E501
    metadata_file_name = "metadata.xml"
    metadata_file_dir = "profiles/default"
    metadata_file_path = (
        configurator.variables["package_folder"]
        + "/"
        + metadata_file_dir
        + "/"
        + metadata_file_name
    )

    with open(metadata_file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        version = tree.xpath("/metadata/version")[0]
        version.text = str(configurator.variables["upgrade_step_dest_version"])

    with open(metadata_file_path, "wb") as xml_file:
        tree.write(
            xml_file,
            pretty_print=True,
            xml_declaration=True,
            encoding="utf-8",
        )


def pre_renderer(configurator):
    """Pre rendering."""
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "upgrade_step"
    upgrade_step_source_version = _read_source_version(configurator)
    upgrade_step_dest_version = upgrade_step_source_version + 1
    configurator.variables["upgrade_step_source_version"] = upgrade_step_source_version
    configurator.variables["upgrade_step_dest_version"] = upgrade_step_dest_version
    configurator.variables["upgrade_step_id"] = str(upgrade_step_dest_version)
    configurator.target_directory = configurator.variables["package_folder"]


def post_renderer(configurator):
    """Post rendering."""
    _update_package_configure_zcml(configurator)
    _update_upgrades_configure_zcml(configurator)
    _write_dest_version(configurator)
    _remove_unwanted_files(configurator)
    git_commit(
        configurator,
        "Add upgrade_step: {0}".format(
            configurator.variables["upgrade_step_title"],
        ),
    )



# -*- coding: utf-8 -*-


class RegEntry(object):
    def __init__(self):
        self.template = ""
        self.plonecli_alias = ""
        self.depend_on = None
        self.deprecated = False
        self.info = ""


def plone_addon():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:addon"
    reg.plonecli_alias = "addon"
    return reg


def plone_buildout():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:buildout"
    reg.plonecli_alias = "buildout"
    return reg


def plone_theme():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:theme"
    reg.plonecli_alias = "theme"
    reg.depend_on = "plone_addon"
    return reg


def plone_theme_barceloneta():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:theme_barceloneta"
    reg.plonecli_alias = "theme_barceloneta"
    reg.depend_on = "plone_addon"
    return reg


def plone_theme_basic():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:theme_basic"
    reg.plonecli_alias = "theme_basic"
    reg.depend_on = "plone_addon"
    return reg


def plone_content_type():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:content_type"
    reg.plonecli_alias = "content_type"
    reg.depend_on = "plone_addon"
    return reg


def plone_view():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:view"
    reg.plonecli_alias = "view"
    reg.depend_on = "plone_addon"
    return reg


def plone_portlet():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:portlet"
    reg.plonecli_alias = "portlet"
    reg.depend_on = "plone_addon"
    return reg


def plone_viewlet():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:viewlet"
    reg.plonecli_alias = "viewlet"
    reg.depend_on = "plone_addon"
    return reg


def plone_vocabulary():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:vocabulary"
    reg.plonecli_alias = "vocabulary"
    reg.depend_on = "plone_addon"
    return reg


def plone_behavior():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:behavior"
    reg.plonecli_alias = "behavior"
    reg.depend_on = "plone_addon"
    return reg


def plone_restapi_service():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:restapi_service"
    reg.plonecli_alias = "restapi_service"
    reg.depend_on = "plone_addon"
    return reg


def plone_svelte_app():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:svelte_app"
    reg.plonecli_alias = "svelte_app"
    reg.depend_on = "plone_addon"
    return reg


def plone_subscriber():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:subscriber"
    reg.plonecli_alias = "subscriber"
    reg.depend_on = "plone_addon"
    return reg


def plone_indexer():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:indexer"
    reg.plonecli_alias = "indexer"
    reg.depend_on = "plone_addon"
    return reg


def plone_upgrade_step():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:upgrade_step"
    reg.plonecli_alias = "upgrade_step"
    reg.depend_on = "plone_addon"
    return reg


def plone_controlpanel():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:controlpanel"
    reg.plonecli_alias = "controlpanel"
    reg.depend_on = "plone_addon"
    return reg


def plone_mockup_pattern():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:mockup_pattern"
    reg.plonecli_alias = "mockup_pattern"
    reg.depend_on = "plone_addon"
    return reg


def plone_form():
    reg = RegEntry()
    reg.template = "bobtemplates.plone:form"
    reg.plonecli_alias = "form"
    reg.depend_on = "plone_addon"
    return reg



# -*- coding: utf-8 -*-

from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import update_file
from bobtemplates.plone.base import ZCML_NAMESPACES
from bobtemplates.plone.utils import run_black
from bobtemplates.plone.utils import run_isort
from lxml import etree

import case_conversion as cc
import os


def _update_package_configure_zcml(configurator):
    file_name = "configure.zcml"
    file_path = configurator.variables["package_folder"] + "/" + file_name

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()
        permid = ".controlpanels"
        xpath_selector = "./include[@package='{0}']".format(permid)  # NOQA: E501
        if len(tree_root.xpath(xpath_selector, namespaces=ZCML_NAMESPACES)):
            print("{0} already in configure.zcml, skip adding!".format(permid))
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
    <include package=".controlpanels" />

"""
    update_file(configurator, file_path, match_str, insert_str)


def _update_controlpanels_configure_zcml(configurator):
    file_name = "configure.zcml"
    file_path = configurator.variables["package_folder"] + "/controlpanels/" + file_name
    example_file_path = file_path + ".example"
    file_list = os.listdir(os.path.dirname(file_path))
    if file_name not in file_list:
        os.rename(example_file_path, file_path)

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()

        xpath_selector = "./include[@package='{0}']".format(
            configurator.variables["controlpanel_name_normalized"],
        )
        if len(tree_root.xpath(xpath_selector, namespaces=ZCML_NAMESPACES)):
            print(
                "{0} already in configure.zcml, skip adding!".format(
                    configurator.variables["controlpanel_name_normalized"]
                )
            )  # NOQA: E501
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
    <include
        package=".{0}"
    />

""".format(
        configurator.variables["controlpanel_name_normalized"],
    )
    update_file(configurator, file_path, match_str, insert_str)


def _update_profile_controlpanel_xml(configurator):
    file_name = "controlpanel.xml"
    file_path = (
        configurator.variables["package_folder"] + "/profiles/default/" + file_name
    )
    example_file_path = file_path + ".example"
    file_list = os.listdir(os.path.dirname(file_path))
    if file_name not in file_list:
        os.rename(example_file_path, file_path)

    with open(file_path, "r") as xml_file:
        parser = etree.XMLParser(remove_blank_text=True)
        tree = etree.parse(xml_file, parser)
        tree_root = tree.getroot()

        xpath_selector = "./configlet[@action_id='{0}-controlpanel']".format(
            configurator.variables["controlpanel_name_normalized"],
        )
        if len(tree_root.xpath(xpath_selector, namespaces=ZCML_NAMESPACES)):
            print(
                "{0} already in {1} skip adding!".format(
                    configurator.variables["controlpanel_name_normalized"],
                    file_name,
                )
            )  # NOQA: E501
            return

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <configlet
      i18n:attributes="title"
      title="{0}"
      action_id="{1}-controlpanel"
      appId="{1}-controlpanel"
      category="Products"
      condition_expr=""
      icon_expr=""
      url_expr="string:${{portal_url}}/@@{1}-controlpanel"
      visible="True">
    <permission>Manage Portal</permission>
  </configlet>

""".format(
        configurator.variables["controlpanel_separated_name"],
        configurator.variables["controlpanel_name_normalized"],
    )
    update_file(configurator, file_path, match_str, insert_str)


def prepare_renderer(configurator):
    configurator = base_prepare_renderer(configurator)
    configurator.variables["template_id"] = "controlpanel"
    controlpanel_name = configurator.variables["controlpanel_python_class_name"].strip(
        "_"
    )
    configurator.variables["controlpanel_name_klass"] = cc.pascalcase(controlpanel_name)
    configurator.variables["controlpanel_name_normalized"] = cc.snakecase(  # NOQA: E501
        controlpanel_name
    )
    configurator.variables["controlpanel_separated_name"] = cc.separate_words(
        controlpanel_name
    )
    configurator.target_directory = configurator.variables["package_folder"]

    # compute the browserlayer like the addon template does
    camelcasename = (
        configurator.variables["package.dottedname"]
        .replace(".", " ")
        .title()
        .replace(" ", "")
        .replace("_", "")
    )
    configurator.variables["package.browserlayer"] = "{0}Layer".format(camelcasename)


def post_renderer(configurator):
    """"""
    _update_package_configure_zcml(configurator)
    _update_controlpanels_configure_zcml(configurator)
    _update_profile_controlpanel_xml(configurator)
    run_isort(configurator)
    run_black(configurator)
    git_commit(
        configurator,
        "Add Control Panel: {0}".format(
            configurator.variables["controlpanel_python_class_name"],
        ),
    )



# -*- coding: utf-8 -*-

from setuptools import find_packages
from setuptools import setup


version = "6.2.4"


long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="bobtemplates.plone",
    version=version,
    description="Templates for Plone projects.",
    long_description=long_description,
    classifiers=[
        "Environment :: Console",
        "Framework :: Plone",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Topic :: Software Development :: Code Generators",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="web plone zope skeleton project",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/bobtemplates.plone/",
    project_urls={
        "PyPI": "https://pypi.python.org/pypi/bobtemplates.plone",
        "Documentation": "https://bobtemplatesplone.readthedocs.io/en/latest/",
        "Source": "https://github.com/plone/bobtemplates.plone/",
        "Tracker": "https://github.com/plone/bobtemplates.plone/issues",
    },
    license="GPL version 2",
    packages=find_packages(exclude=["ez_setup"]),
    namespace_packages=["bobtemplates"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
    install_requires=[
        "setuptools",
        "mr.bob",
        "lxml",
        "case-conversion",
        "colorama",
        "tox",
        "isort",
        "black",
    ],
    setup_requires=[],
    tests_require=[],
    extras_require={},
    entry_points={
        "mrbob_templates": [
            "plone_addon = bobtemplates.plone.bobregistry:plone_addon",
            "plone_behavior = bobtemplates.plone.bobregistry:plone_behavior",
            "plone_buildout = bobtemplates.plone.bobregistry:plone_buildout",
            "plone_content_type = bobtemplates.plone.bobregistry:plone_content_type",
            "plone_indexer = bobtemplates.plone.bobregistry:plone_indexer",
            "plone_mockup_pattern = bobtemplates.plone.bobregistry:plone_mockup_pattern",
            "plone_portlet = bobtemplates.plone.bobregistry:plone_portlet",
            "plone_restapi_service = bobtemplates.plone.bobregistry:plone_restapi_service",  # NOQA E501
            "plone_svelte_app = bobtemplates.plone.bobregistry:plone_svelte_app",
            "plone_subscriber = bobtemplates.plone.bobregistry:plone_subscriber",
            "plone_theme = bobtemplates.plone.bobregistry:plone_theme",
            "plone_theme_barceloneta = bobtemplates.plone.bobregistry:plone_theme_barceloneta",  # NOQA E501
            "plone_theme_basic = bobtemplates.plone.bobregistry:plone_theme_basic",  # NOQA E501
            "plone_upgrade_step = bobtemplates.plone.bobregistry:plone_upgrade_step",
            "plone_view = bobtemplates.plone.bobregistry:plone_view",
            "plone_viewlet = bobtemplates.plone.bobregistry:plone_viewlet",
            "plone_vocabulary = bobtemplates.plone.bobregistry:plone_vocabulary",
            "plone_controlpanel = bobtemplates.plone.bobregistry:plone_controlpanel",
            "plone_form = bobtemplates.plone.bobregistry:plone_form",
        ]
    },
)



# -*- coding: utf-8 -*-

"""Test view generation."""

from bobtemplates.plone import base
from bobtemplates.plone import viewlet
from mrbob.bobexceptions import SkipQuestion
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator
from mrbob.configurator import Question

import os
import pytest


def test_get_view_name_default():
    question = Question(name="view_name", question="", default=None)
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_python_class_name": "DemoViewlet",
        },
    )
    viewlet.get_view_name_from_python_class(configurator, question)


def test_viewlet_template_true():
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_template": True,
        },
    )
    viewlet.check_viewlet_template_answer(configurator, None)


def test_viewlet_template_false():
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.foo.bar",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_template": False,
        },
    )
    with pytest.raises(SkipQuestion):
        viewlet.check_viewlet_template_answer(configurator, None)


def test_update_configure_zcml(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.demo"
    package_path = target_path + "/src/collective/demo"
    os.makedirs(target_path)
    os.makedirs(package_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.demo">

  <i18n:registerTranslations directory="locales" />

  <!--<includeDependencies package="." />-->

  <include package=".viewlets" />

  <include package=".browser" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.demo",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "package_folder": package_path,
        },
    )
    viewlet._update_configure_zcml(configurator)

    with open(os.path.join(package_path + "/configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_update_viewlets_configure_zcml_with_template(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    viewlets_path = package_path + "/viewlets/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(viewlets_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.sample">

  -*- extra stuff goes here -*-
  <browser:viewlet
     name="viewlet"
     for="*"
     manager="plone.app.layout.viewlets.interfaces.IBelowContentTitle"
     layer="collective.sample.interfaces.ICollectiveTodoLayer"
     class=".py_viewlet.MyViewlet"
     template="pt_viewlet.pt"
     permission="zope2.View"
     />


</configure>
"""
    with open(os.path.join(viewlets_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_name": "viewlet",
            "viewlet_python_class_name": "MyViewlet",
            "viewlet_python_file_name": "py_viewlet",
            "viewlet_template": True,
            "viewlet_template_name": "pt_viewlet",
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
            "browser_layer": "ICollectiveSampleLayer",
        },
    )
    viewlet._update_viewlets_configure_zcml(configurator)

    with open(os.path.join(viewlets_path + "configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_update_viewlets_configure_zcml_without_template(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    viewlets_path = package_path + "/viewlets/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(viewlets_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.sample">

  -*- extra stuff goes here -*-
  <browser:viewlet
     name="viewlet"
     for="*"
     manager="plone.app.layout.viewlets.interfaces.IBelowContentTitle"
     layer="collective.sample.interfaces.ICollectiveTodoLayer"
     class=".py_viewlet.MyViewlet"
     permission="zope2.View"
     />


</configure>
"""
    with open(os.path.join(viewlets_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_name": "viewlet",
            "viewlet_python_class_name": "MyViewlet",
            "viewlet_python_file_name": "py_viewlet",
            "viewlet_template": False,
            "package_folder": package_path,
            "package.dottedname": "collective.sample",
            "browser_layer": "ICollectiveSampleLayer",
        },
    )
    viewlet._update_viewlets_configure_zcml(configurator)

    with open(os.path.join(viewlets_path + "configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)


def test_delete_template_file(tmpdir):
    """Test to remove unwanted file."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    viewlets_path = package_path + "/viewlets/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(viewlets_path)
    file_name = "configure.zcml.example"
    file_path = viewlets_path + file_name
    open(file_path, "a").close()
    file_name = "my_viewlet.pt"
    file_path = viewlets_path + file_name
    open(file_path, "a").close()
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_name_normalized": "my_viewlet",
            "viewlet_template": False,
            "viewlet_template_name": "my_viewlet",
            "package_folder": package_path,
        },
    )
    viewlet._delete_unwanted_files(configurator)
    if os.path.isfile(file_path):
        pytest.raises(ValidationError)


def test_delete_template_file_false(tmpdir):
    """Test to not remove template file."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    viewlets_path = package_path + "/viewlets/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(viewlets_path)
    file_name = "configure.zcml.example"
    file_path = viewlets_path + file_name
    open(file_path, "a").close()
    file_name = "my_viewlet.pt"
    file_path = viewlets_path + file_name
    open(file_path, "a").close()
    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory="collective.sample",
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_name_normalized": "my_viewlet",
            "viewlet_template": True,
            "viewlet_template_name": "my_viewlet",
            "package_folder": package_path,
        },
    )
    viewlet._delete_unwanted_files(configurator)
    if not os.path.isfile(file_path):
        pytest.raises(ValidationError)


def test_post_renderer_with_template(tmpdir):
    """Test post rendering."""
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    viewlets_path = package_path + "/viewlets"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(viewlets_path)

    template = """# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import ViewletBase


class MyViewlet(ViewletBase):

    def render(self):
        return u'Sample viewlet!'

"""
    with open(os.path.join(viewlets_path + "/viewlet.py"), "w") as f:
        f.write(template)
    template = """<div class="days_to_conf">
    ${view/get_message}
</div>
"""
    with open(os.path.join(viewlets_path + "/viewlet.pt"), "w") as f:
        f.write(template)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.todo">

    -*- extra stuff goes here -*-
    <browser:viewlet
       name="viewlet"
       for="*"
       manager="plone.app.layout.viewlets.interfaces.IBelowContentTitle"
       layer="collective.todo.interfaces.ICollectiveTodoLayer"
       class=".viewlet.MyViewlet"
       template="viewlet.pt"
       permission="zope2.View"
       />


    </configure>
    """
    with open(os.path.join(viewlets_path + "/configure.zcml"), "w") as f:
        f.write(template)

    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.todo">

  <i18n:registerTranslations directory="locales" />

  <!--
    Be careful if you use general includeDependencies, it can have sideffects!
    Better import explicite packages or configurations ;)
  -->
  <!--<includeDependencies package="." />-->

  <include package=".viewlets" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:viewlet",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_name": "viewlet",
            "viewlet_python_class_name": "MyViewlet",
            "viewlet_template": True,
            "viewlet_template_name": "viewlet",
            "plone.version": "5.1",
        },
    )

    template = """
        dummy
        '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    os.chdir(package_path)
    base.set_global_vars(configurator)
    viewlet.prepare_renderer(configurator)
    viewlet.post_renderer(configurator)


def test_post_renderer_without_template(tmpdir):
    """Test post rendering."""
    target_path = tmpdir.strpath + "/collective.todo"
    package_path = target_path + "/src/collective/todo"
    viewlets_path = package_path + "/viewlets"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(viewlets_path)

    template = """# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import ViewletBase


class MyViewlet(ViewletBase):

    def render(self):
        return u'Sample viewlet!'

"""
    with open(os.path.join(viewlets_path + "/viewlet.py"), "w") as f:
        f.write(template)
    template = """<div class="days_to_conf">
    ${view/get_message}
</div>
"""
    with open(os.path.join(viewlets_path + "/viewlet.pt"), "w") as f:
        f.write(template)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:browser="http://namespaces.zope.org/browser"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.todo">

    -*- extra stuff goes here -*-
    <browser:viewlet
       name="viewlet"
       for="*"
       manager="plone.app.layout.viewlets.interfaces.IBelowContentTitle"
       layer="collective.todo.interfaces.ICollectiveTodoLayer"
       class=".viewlet.MyViewlet"
       template="viewlet.pt"
       permission="zope2.View"
       />


    </configure>
    """
    with open(os.path.join(viewlets_path + "/configure.zcml"), "w") as f:
        f.write(template)

    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:genericsetup="http://namespaces.zope.org/genericsetup"
    xmlns:i18n="http://namespaces.zope.org/i18n"
    xmlns:plone="http://namespaces.plone.org/plone"
    i18n_domain="collective.todo">

  <i18n:registerTranslations directory="locales" />

  <!--
    Be careful if you use general includeDependencies, it can have sideffects!
    Better import explicite packages or configurations ;)
  -->
  <!--<includeDependencies package="." />-->

  <include package=".viewlets" />

</configure>
"""
    with open(os.path.join(package_path + "/configure.zcml"), "w") as f:
        f.write(template)

    configurator = Configurator(
        template="bobtemplates.plone:addon",
        target_directory=package_path,
        bobconfig={
            "non_interactive": True,
        },
        variables={
            "viewlet_name": "viewlet",
            "viewlet_python_class_name": "MyViewlet",
            "viewlet_template": False,
            "plone.version": "5.1",
        },
    )

    template = """
        dummy
        '-*- Extra requirements: -*-'
"""
    with open(os.path.join(target_path + "/setup.py"), "w") as f:
        f.write(template)

    os.chdir(package_path)
    base.set_global_vars(configurator)
    viewlet.prepare_renderer(configurator)
    viewlet.post_renderer(configurator)



# -*- coding: utf-8 -*-

"""Test view generation."""

# from bobtemplates.plone import base
from bobtemplates.plone import subscriber
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator

import os
import pytest


def test_update_subscribers_configure_zcml(tmpdir):
    """Test configure changes when changes are already in place."""
    target_path = tmpdir.strpath + "/collective.sample"
    package_path = target_path + "/src/collective/sample"
    subscribers_path = package_path + "/subscribers/"
    os.makedirs(target_path)
    os.makedirs(package_path)
    os.makedirs(subscribers_path)
    template = """<configure
    xmlns="http://namespaces.zope.org/zope"
    i18n_domain="{{{ package.dottedname }}}">

  -*- extra stuff goes here -*-

  <subscriber for="plone.dexterity.interfaces.IDexterityContent
                   zope.lifecycleevent.interfaces.IObjectModifiedEvent"
              handler=".obj_mod_do_something.handler"
              />

</configure>
"""
    with open(os.path.join(subscribers_path + "configure.zcml"), "w") as f:
        f.write(template)
    configurator = Configurator(
        template="bobtemplates.plone:subscriber",
        target_directory="collective.sample",
        bobconfig={"non_interactive": True},
        variables={
            "subscriber_handler_name": "obj_mod_do_something",
            "subscriber_handler_file_name": "obj_mod_do_something",
            "package_folder": package_path,
        },
    )
    subscriber._update_subscribers_configure_zcml(configurator)

    with open(os.path.join(subscribers_path + "configure.zcml"), "r") as f:
        content = f.read()
        if content != template:
            pytest.raises(ValidationError)



# -*- coding: utf-8 -*-

from .base import init_package_base_structure
from bobtemplates.plone import base
from bobtemplates.plone import theme
from mrbob.bobexceptions import ValidationError
from mrbob.configurator import Configurator
from mrbob.configurator import Question

import os
import pytest


def test_pre_theme_name(tmpdir):
    base_path = tmpdir.strpath
    configurator = Configurator(
        template="bobtemplates.plone:theme",
        target_directory=os.path.join(
            base_path,
            "collective.foo",
        ),
    )
    question = Question(
        "package",
        "type",
    )
    theme.pre_theme_name(configurator, question)
    theme.pre_theme_name(configurator, question)


def test_post_theme_name(tmpdir):
    base_path = tmpdir.strpath
    configurator = Configurator(
        template="bobtemplates.plone:theme",
        target_directory=os.path.join(
            base_path,
            "collective.foo",
        ),
    )

    theme.post_theme_name(configurator, None, "collective.foo")
    with pytest.raises(ValidationError):
        theme.post_theme_name(configurator, None, "collective.$SPAM")


def test_prepare_renderer(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:theme",
        target_directory=package_path,
        variables={
            "theme.name": "My Beautiful Theme (2021)",
        },
    )
    theme.prepare_renderer(configurator)

    assert configurator.variables["template_id"] == "theme"
    assert configurator.variables["theme.normalized_name"] == "my-beautiful-theme-2021"
    assert configurator.target_directory.endswith(
        "/collective.todo/src/collective/todo"
    )  # NOQA: E501

    # nested namespace package
    package_root = os.path.join(
        tmpdir.strpath,
        "collective.foo.bar",
    )
    package_path = init_package_base_structure(package_root)
    configurator = Configurator(
        template="bobtemplates.plone:theme",
        target_directory=package_path,
        variables={
            "theme.name": "My Beautiful Theme (2021)",
            "package.root_folder": package_root,
        },
    )
    theme.prepare_renderer(configurator)

    assert configurator.variables["template_id"] == "theme"
    assert configurator.variables["theme.normalized_name"] == "my-beautiful-theme-2021"
    assert configurator.target_directory.endswith(
        "/collective.foo.bar/src/collective/foo/bar"
    )  # NOQA: E501


def test_post_renderer(tmpdir):
    package_root = tmpdir.strpath + "/collective.todo"
    package_path = init_package_base_structure(package_root)

    configurator = Configurator(
        template="bobtemplates.plone:theme",
        target_directory=package_path,
        bobconfig={"non_interactive": True},
        variables={"plone.version": "5.1", "theme.name": "My Theme"},
    )

    assert configurator
    os.chdir(package_path)
    base.set_global_vars(configurator)
    theme.prepare_renderer(configurator)
    configurator.render()
    theme.post_renderer(configurator)
